@@ -9,13 +9,12 @@ experiment with and understand how command and argument parsing work.
 
 It also serves as an example of how to create shortcuts.
 """
-import argparse
 
 import cmd2
 
 
 class ArgumentAndOptionPrinter(cmd2.Cmd):
-    """ Example cmd2 application where we create commands that just print the arguments they are called with."""
+    """Example cmd2 application where we create commands that just print the arguments they are called with."""
 
     def __init__(self):
         # Create command shortcuts which are typically 1 character abbreviations which can be used in place of a command
@@ -40,7 +39,7 @@ class ArgumentAndOptionPrinter(cmd2.Cmd):
         """Print the argument list this basic command is called with (with quotes preserved)."""
         self.poutput('rprint was called with the following list of arguments: {!r}'.format(arglist))
 
-    oprint_parser = argparse.ArgumentParser()
+    oprint_parser = cmd2.Cmd2ArgumentParser()
     oprint_parser.add_argument('-p', '--piglatin', action='store_true', help='atinLay')
     oprint_parser.add_argument('-s', '--shout', action='store_true', help='N00B EMULATION MODE')
     oprint_parser.add_argument('-r', '--repeat', type=int, help='output [n] times')
@@ -51,12 +50,12 @@ class ArgumentAndOptionPrinter(cmd2.Cmd):
         """Print the options and argument list this options command was called with."""
         self.poutput('oprint was called with the following\n\toptions: {!r}'.format(args))
 
-    pprint_parser = argparse.ArgumentParser()
+    pprint_parser = cmd2.Cmd2ArgumentParser()
     pprint_parser.add_argument('-p', '--piglatin', action='store_true', help='atinLay')
     pprint_parser.add_argument('-s', '--shout', action='store_true', help='N00B EMULATION MODE')
     pprint_parser.add_argument('-r', '--repeat', type=int, help='output [n] times')
 
-    @cmd2.with_argparser_and_unknown_args(pprint_parser)
+    @cmd2.with_argparser(pprint_parser, with_unknown_args=True)
     def do_pprint(self, args, unknown):
         """Print the options and argument list this options command was called with."""
         self.poutput('oprint was called with the following\n\toptions: {!r}\n\targuments: {}'.format(args, unknown))
@@ -64,5 +63,6 @@ class ArgumentAndOptionPrinter(cmd2.Cmd):
 
 if __name__ == '__main__':
     import sys
+
     app = ArgumentAndOptionPrinter()
     sys.exit(app.cmdloop())
